
using namespace std;

class lights
{
	public:
		lights();
		void adjustLightLevel(short mode, short pin, short value);
		void resumeLightMode();
		void sortlightSchedules();
		void calculateLightSchedules();
		void readLightModes();
		void monitorLightSchedules();
		void lightLevelSliderStopped(short mode, short pin, short value);
		void changeLightMode(int modeNum);
		void deleteLightMode(int modeNum);
		void moveLightMode(short dir, short modeN);
		void saveLightMode(int modeNum, int start, int end, char* name, short mon, short tue, short wed, short thu, short fri, short sat, short sun);
		void addLightMode(int modeNum, int start, int end, char* name, short mon, short tue, short wed, short thu, short fri, short sat, short sun);
		void enableLightMode(short modeNum, short val);
	private:
		short lightModeID[30] = {};  // track light mode ID - from lightModes Table
		short lightModeIdSorted[30] = {};  // track light mode ID - from lightModes Table
		short lightModeNumber[30] = {};  // track light mode number - from lightModes Table
		short lightModeNumberSorted[30] = {};  // track light mode number - from lightModes Table
		unsigned long long lightModeStart[30] = {};  // track light mode schedule start time - from lightModes Table
		unsigned long long lightModeStartSorted[30] = {};  // track light mode schedule end time - from lightModes Table
		unsigned long long lightModeEnd[30] = {};  // track light mode schedule end time - from lightModes Table
		unsigned long long lightModeEndSorted[30] = {};  // track light mode schedule end time - from lightModes Table
		bool lightModeMon[30] = {};  // track light mode schedule day - from lightModes Table
		bool lightModeTue[30] = {};  // track light mode schedule day - from lightModes Table
		bool lightModeWed[30] = {};  // track light mode schedule day - from lightModes Table
		bool lightModeThu[30] = {};  // track light mode schedule day - from lightModes Table
		bool lightModeFri[30] = {};  // track light mode schedule day - from lightModes Table
		bool lightModeSat[30] = {};  // track light mode schedule day - from lightModes Table
		bool lightModeSun[30] = {};  // track light mode schedule day - from lightModes Table
		bool days[30][7] = {};  // 
		bool daysSorted[30][7] = {};  // 
		short lightModeEnabled[30] = {};  // track if light mode is enabled - from lightModes Table
		short lightModePinID[30][50] = {};  // track light mode pin number - from lightModeDetails Table  -- up to 100 modes and 50 channels (3 pca9685 + 2 Pi PWM pins)
		short lightModePinNumber[30][50] = {};  // track light mode pin number - from lightModeDetails Table
		short lightModeLevels[30][50] = {};  // track channel level for each mode - from lightModeDetails Table
		short lightModeLevelsSorted[30][50] = {};  // track channel level for each mode - from lightModeDetails Table
		short totalLightModes = 0;  // track total number of light modes
		short totalLightChannels = 0;  // track total number of channels
		bool fadeRunning = 0;  // track if fade is running
		bool testFade = 0;  // track if we are testing the fade
		unsigned long long nextFadeStart = 0;  // track next light mode start time
		short nextLightModeID = 0;  // next light mode ID
		short nextLightModeNumber = 0;  // next light mode number
		unsigned long long currentFadeDuration = 0;  // track current light mode fade duration
		unsigned long long currentFadeEnd = 0;  // track current light mode end time
		short currentFadeAmount[30] = {};  // track how many levels channel needs to fade
		short currentLightModeID = 0;  // current light mode ID
		short currentLightModeNumber = 0;  // current light mode number
		short previousLightModeNumber = 0;
		short lightsStartupComplete = 0;  // track when light mode data is know so we can monitor for fades
		bool pauseFading = 0;
		short prevFadeValue[30] = {};  // tracks when the fade value changes so we only send command to pca9685 when needed
		short updatingLightModes = 0;  // track when a new light mode is being added or removed so monitoring can be paused
};
lights::lights(){}

void lights::adjustLightLevel(short mode, short pinID, short value) {  // adjust PWM levels for each channel
    // std::cout << "mode  - " << mode << std::endl;
    // std::cout << "pin   - " << pin << std::endl;
    // std::cout << "value - " << value << std::endl;
	int arrayLoc = 0;
	for (int b=0; b < totalLightChannels; b++)  // find channel number based on pin number
	{
		if (lightModePinID[mode][b] == pinID) {break;}
		else {arrayLoc++;}
	}
		
	lightModeLevels[mode][arrayLoc] = value;  // store new value
	pca9685.adjustPinLevel(pinID, value);  // adjust pca9685 pin
}

void lights::resumeLightMode() {  // resume light levels to current light mode
    // std::cout << "resumeLightMode  - " << currentLightModeNumber << endl;
	if (pauseFading == 1) pauseFading = 0;
	for (int a=0; a < totalLightChannels; a++)  // 
	{
		adjustLightLevel(currentLightModeNumber,lightModePinID[currentLightModeNumber][a],lightModeLevels[currentLightModeNumber][a]);
	}
}

void lights::sortlightSchedules() {  // sort light schedules by start time
    // std::cout << "sortlightSchedules  --------- " << endl;

	for(short n=0; n < totalLightModes; n++) {  // make copy of original arrays for sorting
		lightModeStartSorted[n] = lightModeStart[n];
		lightModeEndSorted[n] = lightModeEnd[n];
		lightModeIdSorted[n] = lightModeID[n];
		lightModeNumberSorted[n] = lightModeNumber[n];
		for (int a=0; a < 7; a++) {
			daysSorted[n][a] = days[n][a];
		}
		for (int a=0; a < totalLightChannels; a++) {
			lightModeLevelsSorted[n][a] = lightModeLevels[n][a];
		}
	}

	for(short n=0; n < (totalLightModes-1); n++) {  // sort arrays together
		for(short o=0; o < (totalLightModes-(n+1)); o++) {
			if(lightModeStartSorted[o] > lightModeStartSorted[o+1]) {
				unsigned long long aa = lightModeStartSorted[o];
				lightModeStartSorted[o] = lightModeStartSorted[o+1];
				lightModeStartSorted[o+1] = aa;

				unsigned long long bb = lightModeEndSorted[o];
				lightModeEndSorted[o] = lightModeEndSorted[o+1];
				lightModeEndSorted[o+1] = bb;
				
				short cc = lightModeIdSorted[o];
				lightModeIdSorted[o] = lightModeIdSorted[o+1];
				lightModeIdSorted[o+1] = cc;
				
				short dd = lightModeNumberSorted[o];
				lightModeNumberSorted[o] = lightModeNumberSorted[o+1];
				lightModeNumberSorted[o+1] = dd;
				
				for (int a=0; a < 7; a++) {
					short bb = daysSorted[o][a];
					daysSorted[o][a] = daysSorted[o+1][a];
					daysSorted[o+1][a] = bb;
				}

				for (int a=0; a < totalLightChannels; a++) {
					short bb = lightModeLevelsSorted[o][a];
					lightModeLevelsSorted[o][a] = lightModeLevelsSorted[o+1][a];
					lightModeLevelsSorted[o+1][a] = bb;
				}
				
			}
		}
	}
	// for(short n=0; n < totalLightModes; n++) {  // print to console
    // 	std::cout << "o - " << lightModeID[n] << " - " << lightModeNumber[n] << " - " << lightModeStart[n] << " - " << lightModeEnd[n] << " - ";
	// 	for (int a=0; a < 7; a++)  // 
	// 	{
	// 		std::cout << days[n][a] << ", ";
	// 	}
	// 	std::cout << "" << std::endl;

	// }
	// for(short n=0; n < totalLightModes; n++) {  // print to console
    // 	std::cout << "s - " << lightModeIdSorted[n] << " - " << lightModeNumberSorted[n] << " - " << lightModeStartSorted[n] << " - " << lightModeEndSorted[n] << " - ";
	// 	for (int a=0; a < 7; a++)  // 
	// 	{
	// 		std::cout << daysSorted[n][a] << ", ";
	// 	}
	// 	std::cout << "" << std::endl;
	// }
    // std::cout << "sortlightSchedules  --------- " << endl;
}

void lights::calculateLightSchedules() {  // find previous, current and next light modes
	sortlightSchedules();

    // std::cout << "-currentMillisTime     - " << currentMillisTime << std::endl;
	// FIND CURRENT LIGHT MODE NUMBER  --------------------------------------------------------------------------------------------------------------------
	short searchDay = currentWeekday;  // used to search previous week for current schedule
	short f = 0;  // flag if schedule is found
	unsigned long long startSortedTemp[30] = {};  // track light mode schedule end time - from lightModes Table
	unsigned long long endSortedTemp[30] = {};  // track light mode schedule end time - from lightModes Table
	unsigned long long startSortedTemp2[30] = {};  // track light mode schedule end time - from lightModes Table

    	// std::cout << "midnightMillis         - " << midnightMillis << std::endl;
	for(short a=0; a < totalLightModes; a++) {  // go through each light mode
		if (lightModeStartSorted[a] <= lightModeEndSorted[a])  // Light mode starts and ends before midnight
		{
			endSortedTemp[a] = lightModeEndSorted[a] + midnightMillis;  // convert schedule millisconds to timestamp
			lightModeEndSorted[a] = lightModeEndSorted[a] + midnightMillis;  // convert schedule millisconds to timestamp
		}
		else  // light mode starts before midnight but ends after midnight
		{
			endSortedTemp[a] = lightModeEndSorted[a] + midnightMillis + 86400000; // add 24 hours to end time
			lightModeEndSorted[a] = lightModeEndSorted[a] + midnightMillis + 86400000; // add 24 hours to end time
		}
		startSortedTemp[a] = lightModeStartSorted[a] + midnightMillis;  // convert schedule millisconds to timestamp
		startSortedTemp2[a] = lightModeStartSorted[a] + midnightMillis;  // convert schedule millisconds to timestamp
		lightModeStartSorted[a] = lightModeStartSorted[a] + midnightMillis;  // convert schedule millisconds to timestamp
	}
	for(short d=0; d < 7; d++) {  // go through a previous week until current schedule is found
		if (f == 1) break;  // schedule found, stop search
		searchDay = currentWeekday-d;  // subtract day and continue search
		if (searchDay < 0) searchDay = searchDay + 7;  // start week over
		for(short a=1; a < totalLightModes+1; a++) {  // go through each light mode
			if (lightModeEnabled[totalLightModes-a] == 1) {  // include mode in search if enabled
				if ((daysSorted[totalLightModes-a][searchDay] == 1) && ((currentMillisTime > startSortedTemp[totalLightModes-a]) && (currentMillisTime < lightModeEndSorted[totalLightModes-a])))  // found current schedule
				{
					currentFadeDuration = endSortedTemp[totalLightModes-a] - startSortedTemp[totalLightModes-a];
					currentFadeEnd = endSortedTemp[totalLightModes-a];

					currentLightModeID = lightModeIdSorted[totalLightModes-a];
					currentLightModeNumber = lightModeNumberSorted[totalLightModes-a];
					f = 1;  // flag current schedule is found so search can end
					break;
				}
				if ((daysSorted[totalLightModes-a][searchDay] == 1) && (currentMillisTime > endSortedTemp[totalLightModes-a]))  // found current schedule
				{
					currentFadeDuration = endSortedTemp[totalLightModes-a] - startSortedTemp[totalLightModes-a];
					currentFadeEnd = endSortedTemp[totalLightModes-a];

					currentLightModeID = lightModeIdSorted[totalLightModes-a];
					currentLightModeNumber = lightModeNumberSorted[totalLightModes-a];
					f = 1;  // flag current schedule is found so search can end
					break;
				}
				startSortedTemp[totalLightModes-a] = startSortedTemp[totalLightModes-a] - 86400000;
				endSortedTemp[totalLightModes-a] = endSortedTemp[totalLightModes-a] - 86400000;
			}
		}
	}

	// assemble the frontend DIV ID so we can use in frontend 
	std::string currentSch = "";
	if (searchDay == 0) {currentSch = "lightSun_" + to_string(currentLightModeID) + "";}
	else if (searchDay == 1) {currentSch = "lightMon_" + to_string(currentLightModeID) + "";}
	else if (searchDay == 2) {currentSch = "lightTue_" + to_string(currentLightModeID) + "";}
	else if (searchDay == 3) {currentSch = "lightWed_" + to_string(currentLightModeID) + "";}
	else if (searchDay == 4) {currentSch = "lightThu_" + to_string(currentLightModeID) + "";}
	else if (searchDay == 5) {currentSch = "lightFri_" + to_string(currentLightModeID) + "";}
	else if (searchDay == 6) {currentSch = "lightSat_" + to_string(currentLightModeID) + "";}


	// FIND PREVIOUS LIGHT MODE NUMBER  --------------------------------------------------------------------------------------------------------------------
	// searchDay = currentWeekday;  // used to search previous week for current schedule
	// f = 0;  // flag if schedule is found
	// for(short d=0; d < 7; d++) {  // go through a previous week until current schedule is found
	// 	if (f == 1) break;  // schedule found, stop search
			
	// 	for(short a=1; a < totalLightModes+1; a++) {  // go through each schedule
	// 		if (lightModeEnabled[totalLightModes-a] == 1) {  // include mode in search if enabled
	// 			if ((daysSorted[totalLightModes-a][searchDay] == 1) && (currentLightModeNumber != lightModeNumberSorted[totalLightModes-a]))  // found current schedule
	// 			{
	// 				previousLightModeNumber = lightModeNumberSorted[totalLightModes-a];
	// 				f = 1;  // flag current schedule is found so search can end
	// 				break;
	// 			}
	// 		}
	// 	}
	// 	searchDay = currentWeekday-d-1;  // subtract day and continue search
	// 	if (searchDay < 0) searchDay = searchDay + 7;  // start week over
	// }
 
 

    	// std::cout << "FIND PREVIOUS LIGHT MODE NUMBER  - " << std::endl;  
	// FIND PREVIOUS LIGHT MODE NUMBER  --------------------------------------------------------------------------------------------------------------------
	searchDay = currentWeekday;  // used to search previous week for current schedule
	f = 0;  // flag if schedule is found
	for(short d=0; d < 7; d++) {  // go through a previous week until current schedule is found
		if (f == 1) break;  // schedule found, stop search
			
    	// std::cout << "searchDay               - " << searchDay << std::endl;  
		for(short a=1; a < totalLightModes+1; a++) {  // go through each schedule
			if (lightModeEnabled[totalLightModes-a] == 1) {  // include mode in search if enabled
    	// std::cout << "startSortedTemp2[totalLightModes-a] - " << startSortedTemp2[totalLightModes-a] << std::endl;  
				if ((daysSorted[totalLightModes-a][searchDay] == 1) && (currentFadeEnd > startSortedTemp2[totalLightModes-a]) && (currentLightModeNumber != lightModeNumberSorted[totalLightModes-a]))  // found current schedule
				{
					previousLightModeNumber = lightModeNumberSorted[totalLightModes-a];
					f = 1;  // flag current schedule is found so search can end
					break;
				}
				startSortedTemp2[totalLightModes-a] = startSortedTemp2[totalLightModes-a] - 86400000;
			}
		}
		searchDay = currentWeekday-d-1;  // subtract day and continue search
    	// std::cout << "searchDay2              - " << searchDay << std::endl;  
		if (searchDay < 0) searchDay = searchDay + 7;  // start week over
    	// std::cout << "searchDay3              - " << searchDay << std::endl;  
	}
    	// std::cout << "" << std::endl;  
	

	// FIND NEXT LIGHT MODE NUMBER  --------------------------------------------------------------------------------------------------------------------
	searchDay = currentWeekday;  // used to search previous week for current schedule
	f = 0;  // flag if schedule is found
	for(short d=0; d < 7; d++) {  // go through a previous week until current schedule is found
		if (f == 1) break;  // schedule found, stop search
    	// std::cout << "searchDay               - " << searchDay << std::endl;  
		searchDay = currentWeekday+d;  // add day
    	// std::cout << "searchDay2              - " << searchDay << std::endl;  
		if (searchDay > 6) searchDay = searchDay - 7;  // start week over
    	// std::cout << "currentMillisTime       - " << currentMillisTime << std::endl;

		for(short a=0; a < totalLightModes; a++) {  //  
			if (lightModeEnabled[a] == 1) {  // include mode in search if enabled
    			// std::cout << "lightModeStartSorted[a] - " << lightModeStartSorted[a] << std::endl;
				if ((daysSorted[a][searchDay] == 1) && (lightModeStartSorted[a] > currentMillisTime))  // found schedule for today
				{
					nextFadeStart = lightModeStartSorted[a];
					nextLightModeID = lightModeIdSorted[a];
					nextLightModeNumber = lightModeNumberSorted[a];
					f = 1;  // flag current schedule is found so search can end
					break;
				}
				lightModeStartSorted[a] = lightModeStartSorted[a] + 86400000;  // add day to schedule start times
			}
		}
	}

	// assemble the frontend DIV ID so we can use in frontend 
	std::string nextSch = "";
	if (searchDay == 0) {nextSch = "lightSun_" + to_string(nextLightModeID) + "";}
	else if (searchDay == 1) {nextSch = "lightMon_" + to_string(nextLightModeID) + "";}
	else if (searchDay == 2) {nextSch = "lightTue_" + to_string(nextLightModeID) + "";}
	else if (searchDay == 3) {nextSch = "lightWed_" + to_string(nextLightModeID) + "";}
	else if (searchDay == 4) {nextSch = "lightThu_" + to_string(nextLightModeID) + "";}
	else if (searchDay == 5) {nextSch = "lightFri_" + to_string(nextLightModeID) + "";}
	else if (searchDay == 6) {nextSch = "lightSat_" + to_string(nextLightModeID) + "";}
		
	// save modes to database for frontend
	connectMysql();
	std::string query = "UPDATE frontEndData SET nextLightSchedule = '" + nextSch + "', currentLightSchedule = '" + currentSch + "'"; 
	mysql_query(conn, query.c_str());  // run query
	mysql_disconnect(); 

    // std::cout << " " << endl;
    // std::cout << "calculate light modes  ---------------- " << endl;
    // std::cout << "currentSch DIV           - " << currentSch << std::endl;  
    // std::cout << "nextSch DIV              - " << nextSch << std::endl;  
    // std::cout << "currentFadeEnd           - " << currentFadeEnd << std::endl;
    // std::cout << "nextFadeStart            - " << nextFadeStart << std::endl;
    // std::cout << "currentFadeDuration      - " << currentFadeDuration << std::endl;
    // std::cout << "previousLightModeNumber  - " << previousLightModeNumber << std::endl;
    // std::cout << "currentLightModeID       - " << currentLightModeID << std::endl;
    // std::cout << "currentLightModeNumber   - " << currentLightModeNumber << std::endl;
    // std::cout << "nextLightModeID          - " << nextLightModeID << std::endl;
    // std::cout << "nextLightModeNumber      - " << nextLightModeNumber << std::endl;
    // std::cout << " " << endl;

	if (fadeRunning == 0) {resumeLightMode();}
	
	updatingLightModes = 0;
}

void lights::readLightModes() {  // read light modes from database
    std::cout << " " << std::endl;
    std::cout << "readLightModes--------------------------------" << std::endl;
	updatingLightModes = 1;
    MYSQL_RES *res;
    MYSQL_ROW row;
	
	// search database for any pins set as light
    connectMysql();
    std::string query = "SELECT pinID FROM pinsAvailable WHERE pinTypeID = '11' AND assigned = 'Yes'";  // construct query string
    mysql_query(conn, query.c_str());  // run query

    res = mysql_use_result(conn);

    std::fill_n(lightModeID, 30, 0);  // reset array
    std::fill_n(lightModeNumber, 30, 0);  // reset array
    std::fill_n(lightModeStart, 30, 0);  // reset array
    std::fill_n(lightModeEnd, 30, 0);  // reset array
    // std::fill_n(days, 30, 0);  // reset array
    std::fill_n(lightModeEnabled, 30, 0);  // reset array
	totalLightChannels = 0;
    while ((row = mysql_fetch_row(res)) != NULL)
    {
	    totalLightChannels++;
    }
    mysql_disconnect(); 
    mysql_free_result(res);

    // std::cout << "totalLightChannels - " << totalLightChannels << std::endl;

    connectMysql();
    query = "SELECT modeID, modeNumber, start, end, sun, mon, tue, wed, thu, fri, sat, enabled FROM lightModes ORDER BY modeNumber ASC";  // construct query string
    mysql_query(conn, query.c_str());  // run query

    res = mysql_use_result(conn);
	totalLightModes = 0;
	short i = 0;
    while ((row = mysql_fetch_row(res)) != NULL)
    {
		lightModeID[i] = atoi(row[0]);
		lightModeNumber[i] = atoi(row[1]);
		lightModeStart[i] = atoi(row[2]) * 1000;  // convert from seconds to milliseconds
		lightModeEnd[i] = atoi(row[3]) * 1000;  // convert from seconds to milliseconds

		for (int a=0; a < 7; a++)  // 
		{
			days[i][a] = atoi(row[4+a]);
		}
		lightModeEnabled[i] = atoi(row[11]);
		totalLightModes++;
		i++;
    }
    mysql_disconnect(); 
    mysql_free_result(res);

	for(short n=0; n < totalLightModes; n++) {  // print to console
    	std::cout << "o - " << lightModeID[n] << " - " << lightModeNumber[n] << " - " << lightModeStart[n] << " - " << lightModeEnd[n] << " - ";
		for (int a=0; a < 7; a++)  // 
		{
			std::cout << days[n][a] << ", ";
		}
		std::cout << "" << std::endl;
	}
	
	std::fill_n(&lightModePinID[0][0], 30 * 50, false);
	std::fill_n(&lightModePinNumber[0][0], 30 * 50, false);
	std::fill_n(&lightModeLevels[0][0], 30 * 50, false);
	
	query = "SELECT lightModeDetails.modeID, lightModeDetails.pinID, lightModeDetails.lightLevel, lightModes.modeNumber, pinsAvailable.wiringpi FROM lightModeDetails LEFT JOIN lightModes ON lightModeDetails.modeID = lightModes.modeID LEFT JOIN pinsAvailable ON lightModeDetails.pinID = pinsAvailable.pinID ORDER BY lightModes.modeNumber, lightModeDetails.pinID ASC";
    connectMysql();
	mysql_query(conn, query.c_str());  // run query
    res = mysql_use_result(conn);

	for (int a=0; a < totalLightModes; a++)  // 
	{
        // std::cout << "modeID - " << lightModeID[a] << std::endl;
        // std::cout << "modeNumber - " << lightModeNumber[a] << std::endl;
		for (int b=0; b < totalLightChannels; b++)  // 
		{
			row = mysql_fetch_row(res);
			lightModePinID[lightModeNumber[a]][b] = atoi(row[1]);
			lightModePinNumber[lightModeNumber[a]][b] = atoi(row[4]);
			lightModeLevels[lightModeNumber[a]][b] = atoi(row[2]);
		}
	}
    mysql_free_result(res);
    mysql_disconnect(); 
	// delay(100);

	calculateLightSchedules();

	lightsStartupComplete = 1;
	// int wake 	= settings & 0xEF;									// Set sleep bit to 0
	
    // char buffer[2];

    // buffer[0] = 0x00; 
    // buffer[1] = 0xEF; 

    // write(pca9685I2Cfile[0], buffer, 2); 

}

void lights::monitorLightSchedules()  // continually monitor schedules
{
	if (updatingLightModes == 0)
	{
		if ((lightsStartupComplete == 1) && (totalLightModes > 0)) {
			if (((currentFadeEnd > currentMillisTime) || (currentMillisTime > nextFadeStart)) && (fadeRunning == 0)) {  //  
				// std::cout << "monitorLightSchedules --------- " << std::endl;
				// std::cout << "fading is running for mode number -  " << currentLightModeNumber << std::endl;
				// std::cout << "previousLightModeNumber           - " << previousLightModeNumber << std::endl;
				// std::cout << "currentFadeEnd                    - " << currentFadeEnd << std::endl;

				fadeRunning = 1;
				calculateLightSchedules();
				sleepTimeMainLoop = 0.1;
				// std::cout << "currentFadeAmount = " << std::endl;
				for (short a=0; a < totalLightChannels; a++)  // calculate how much each channel needs to fade
				{
					currentFadeAmount[a] = lightModeLevelsSorted[currentLightModeNumber][a] - lightModeLevelsSorted[previousLightModeNumber][a];
					// std::cout << "    " << currentFadeAmount[a] << std::endl;
				}
			}

			if ((fadeRunning == 1) || (testFade >= 1)) {
				short v = 0;  // led value
				float f = 0;  // fade level
				float e = 0;  // elapsed time
				if ((currentMillisTime <= currentFadeEnd) && (pauseFading == 0))  // fade running
				{
					for (short a=0; a < totalLightChannels; a++)  // fade each channel
					{
						e = currentFadeEnd - currentMillisTime;
						f = e / currentFadeDuration;
						v = lightModeLevelsSorted[currentLightModeNumber][a] - (currentFadeAmount[a] * f);  // calculate value 
						// std::cout << "v - " << v << std::endl;

						if (prevFadeValue[a] != v) {  // check if value changed since we updated pca9685
							adjustLightLevel(currentLightModeNumber,lightModePinID[currentLightModeNumber][a],v);
							health = health + 50;
							prevFadeValue[a] = v;  // store updated value
							// if (a == 3) std::cout << "val - " << v << " - ch - " << a << std::endl;
						}
					}
				}
				else if ((fadeRunning == 1) && (pauseFading == 0))  // end of fade
				{
					// std::cout << "FADE OVER --------- " << std::endl;
					// readLightModes();
					calculateLightSchedules();
					fadeRunning = 0;
					sleepTimeMainLoop = 1;
				}
			}
		}
	}
}

void lights::lightLevelSliderStopped(short mode, short pin, short value) {  // only save light level when slider stops moving
	adjustLightLevel(mode, pin, value);
	
    // std::cout << "lightLevelSliderStopped  - " << endl;
    // std::cout << "mode         - " << mode << std::endl;
    // std::cout << "lightModeID  - " << lightModeID[mode] << std::endl;
    // std::cout << "pin	      - " << pin << std::endl;
    // std::cout << "value		  - " << value << std::endl;
    connectMysql();
    std::string query = "UPDATE lightModeDetails SET lightLevel = '" + to_string(value) + "' WHERE modeID = '" + to_string(lightModeID[mode]) + "' AND pinID = '" + to_string(pin) + "'";
    mysql_query(conn, query.c_str());  // run query
    mysql_disconnect(); 
}

void lights::changeLightMode(int modeNum) {  // change light modes
	pauseFading = 1;
    // std::cout << "pauseFading - " << pauseFading << std::endl;
	for (int a=0; a < totalLightChannels; a++)  // 
	{
		adjustLightLevel(lightModeNumber[modeNum],lightModePinID[modeNum][a],lightModeLevels[modeNum][a]);
	}
}

void lights::deleteLightMode(int modeNum) {  // delete light mode
    // std::cout << "deleteLightMode  - " << endl;
	fadeRunning = 0;
	updatingLightModes = 1;
    connectMysql();
    std::string query = "DELETE FROM lightModes WHERE modeNumber = '" + to_string(modeNum) + "'";
    mysql_query(conn, query.c_str());  // run query
    mysql_disconnect(); 

	readLightModes();
    // std::cout << "totalLightModes    - " << totalLightModes << std::endl;

	updatingLightModes = 1;
	// renumber remaining light mode numbers
	for (int a=0; a < totalLightModes; a++)  // 
	{
        lightModeNumber[a] = a;  // change light mode number 
		connectMysql();
    	query = "UPDATE lightModes SET modeNumber = '" + to_string(lightModeNumber[a]) + "' WHERE modeID = '" + to_string(lightModeID[a]) + "'";
		mysql_query(conn, query.c_str());  // run query
		mysql_disconnect(); 
	}
	readLightModes();
}

void lights::moveLightMode(short dir, short modeN) {  // shif save updated light mode numbers
    // std::cout << "moveLightMode  - " << endl;
    // std::cout << "dir    - " << dir << std::endl;
    // std::cout << "modeN  - " << modeN << std::endl;
    // mode number is array location
	short newModeNumber = 0;  // track new mode number
	
    if (dir == 0) {newModeNumber = modeN - 1;}  // move up
    else {newModeNumber = modeN + 1;}  // move down

	if ((newModeNumber >= 0) && (newModeNumber < totalLightModes)) {  // check if we aren't moving 1st item up
		// swap mode numbers
		lightModeNumber[modeN] = newModeNumber;
		lightModeNumber[newModeNumber] = modeN;

		for (int a=0; a < totalLightModes; a++)  // save updated light mode numbers
		{
			connectMysql();
			std::string query = "UPDATE lightModes SET modeNumber = '" + to_string(lightModeNumber[a]) + "' WHERE modeID = '" + to_string(lightModeID[a]) + "'";
			mysql_query(conn, query.c_str());  // run query
			mysql_disconnect(); 
		}
	}
	readLightModes();
}

void lights::saveLightMode(int modeNum, int start, int end, char* name, short mon, short tue, short wed, short thu, short fri, short sat, short sun) {  // update existing light mode
    // std::cout << "saveLightMode----------- " << endl;
    // std::cout << "modeNum  - " << modeNum << std::endl;
    // std::cout << "start    - " << start << std::endl;
    // std::cout << "end      - " << end << std::endl;
    // std::cout << "name     - " << name << std::endl;
    // std::cout << "mon      - " << mon << std::endl;
    // std::cout << "tue      - " << tue << std::endl;
    // std::cout << "wed      - " << wed << std::endl;
    // std::cout << "thu      - " << thu << std::endl;
    // std::cout << "fri      - " << fri << std::endl;
    // std::cout << "sat      - " << sat << std::endl;
    // std::cout << "sun      - " << sun << std::endl;
	
    connectMysql();
    std::string query = "UPDATE lightModes SET start = '" + to_string(start) + "', end = '" + to_string(end) + "', modeName = '" + name + "', mon = '" + to_string(mon) + "', tue = '" + to_string(tue) + "', wed = '" + to_string(wed) + "', thu = '" + to_string(thu) + "', fri = '" + to_string(fri) + "', sat = '" + to_string(sat) + "', sun = '" + to_string(sun) + "' WHERE modeNumber = '" + to_string(modeNum) + "'";
    mysql_query(conn, query.c_str());  // run query
    mysql_disconnect(); 

	readLightModes();
}

void lights::addLightMode(int modeNum, int start, int end, char* name, short mon, short tue, short wed, short thu, short fri, short sat, short sun) {  // save new light mode
    // std::cout << "addLightMode  - " << endl;
    // std::cout << "modeNum  - " << modeNum << std::endl;
    // std::cout << "start  - " << start << std::endl;
    // std::cout << "end  - " << end << std::endl;
    // std::cout << "name  - " << name << std::endl;
    // std::cout << "mon      - " << mon << std::endl;
    // std::cout << "tue      - " << tue << std::endl;
    // std::cout << "wed      - " << wed << std::endl;
    // std::cout << "thu      - " << thu << std::endl;
    // std::cout << "fri      - " << fri << std::endl;
    // std::cout << "sat      - " << sat << std::endl;
    // std::cout << "sun      - " << sun << std::endl;
	
    MYSQL_RES *res;
    MYSQL_ROW row;

	// add new light mode to database
	connectMysql();
	std::string query = "INSERT INTO lightModes(modeNumber,modeName,start,end,mon,tue,wed,thu,fri,sat,sun) VALUES('" + to_string(modeNum) + "','" + name + "','" + to_string(start) + "','" + to_string(end) + "','" + to_string(mon) + "','" + to_string(tue) + "','" + to_string(wed) + "','" + to_string(thu) + "','" + to_string(fri) + "','" + to_string(sat) + "','" + to_string(sun) + "')";
	mysql_query(conn, query.c_str());  // run query
	mysql_disconnect(); 

	// get modeID for new mode just added
	connectMysql();
	query = "SELECT modeID FROM lightModes WHERE modeNumber = '" + to_string(modeNum) + "'";
    mysql_query(conn, query.c_str());  // run query
    res = mysql_use_result(conn);
	int ID = 0;
    while ((row = mysql_fetch_row(res)) != NULL)
    {
		ID = atoi(row[0]);
	}
    mysql_disconnect(); 
    mysql_free_result(res);
	
	// get LED pin numbers
	connectMysql();
	query = "SELECT pinID FROM pinsAvailable WHERE pinTypeID = 11 AND assigned = 'Yes'";
    mysql_query(conn, query.c_str());  // run query
    res = mysql_use_result(conn);
	int pin[50] = {};
	int i = 0;
    while ((row = mysql_fetch_row(res)) != NULL)
    {
		pin[i] = atoi(row[0]);
		i++;
	}
    mysql_free_result(res);
    mysql_disconnect(); 
	
	// add LED channels to database
	for (int a=0; a < i; a++)  // 
	{
		connectMysql();
		query = "INSERT INTO lightModeDetails(modeID, pinID, lightLevel) VALUES('" + to_string(ID) + "','" + to_string(pin[a]) + "','0')";
		mysql_query(conn, query.c_str());  // run query
    	mysql_disconnect(); 
	}

	readLightModes();
}

void lights::enableLightMode(short modeNum, short val) {  // enable/disable light mode
    // std::cout << "enableLightMode  - " << endl;
	lightModeEnabled[modeNum] = val;

	// if (val == 0) {setPWMFreq(50);}
	// else {setPWMFreq(1000);}


    connectMysql();
    std::string query = "UPDATE lightModes SET enabled = '" + to_string(lightModeEnabled[modeNum]) + "' WHERE modeNumber = '" + to_string(modeNum) + "'";
    mysql_query(conn, query.c_str());  // run query
    mysql_disconnect(); 

	readLightModes();
}

lights lights;

