
using namespace std;

bool isStringNumbers(const std::string &str)
{
    return str.find_first_not_of("0123456789") == std::string::npos;
}
struct tm result;
time_t t = time(NULL);
unsigned long long convertedMillis = 0;
short nextCurrentDay = 0;
short nextCurrentMonth = 0;
short nextCurrentYear = 0;
short currentDay = 0;
short currentMonth = 0;
short currentYear = 0;
short currentWeekday = 0;

class systemSettings
{
    public:
        systemSettings();
        void logError(short errorSourceID, std::string sensor, std::string msg);
        void updateTimeFormat(short format);
        unsigned long long tmConvert_t(int YY, short MM, short DD, short hh, short mm, short ss);
        void syncDateTimeError();
        void syncDateTimeStartup();
        void syncDateTime();
        void updateTime();
        void restart();  // restart controller
        std::string format_number(int num, int l);
    private:
        unsigned long long currentMillisLastSync = 0;
        unsigned long long prevTimeSyncMillis = 0;
};
systemSettings::systemSettings(){}


void systemSettings::logError(short errorSourceID, std::string sensor, std::string msg) {
	// std::cout << "logError-------------" << endl;
	// add error to database
	connectMysql();
	std::string query = "INSERT INTO errors(errorSourceID,sensor,message) VALUES('" + to_string(errorSourceID) + "','" + sensor + "','" + msg + "')";
	mysql_query(conn, query.c_str());  // run query
	mysql_disconnect(); 
}

void systemSettings::updateTimeFormat(short format) {
    timeFormat = format;
    connectMysql();
    std::string query = "UPDATE systemSettings SET timeFormat = '" + to_string(timeFormat) + "'";
    mysql_query(conn, query.c_str());  // run query
    mysql_disconnect(); 
}

unsigned long long systemSettings::tmConvert_t(int YY, short MM, short DD, short hh, short mm, short ss) {
    result.tm_sec = ss;
    result.tm_min = mm;
    result.tm_hour = hh;
    result.tm_year = YY - 1900;
    result.tm_mon = MM;
    result.tm_mday = DD;
    convertedMillis = std::mktime(&result);
    convertedMillis = convertedMillis * 1000;
    return convertedMillis; 
}

void systemSettings::updateTime() {
    currentMillisTime = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();

    localtime_r(&t, &result);
    currentDay = result.tm_mday;  // 1-31
    currentMonth = result.tm_mon;  // 0 = January
    currentYear = result.tm_year + 1900;
    currentWeekday = result.tm_wday;  // 0 = Sunday
    
    midnightMillis = tmConvert_t(currentYear,currentMonth,currentDay,0,0,00);

    // // used to calculate next midnight millis
    nextCurrentDay = currentDay + 1;
    nextCurrentMonth = currentMonth;
    nextCurrentYear = currentYear;

    // if month has 31 days change next day and month to next month
    if (((currentMonth == 0) || (currentMonth == 2) || (currentMonth == 4) || (currentMonth == 6) || (currentMonth == 7) || (currentMonth == 9) || (currentMonth == 11)) && (nextCurrentDay == 32))
    {
        nextCurrentDay = 1;
        nextCurrentMonth = nextCurrentMonth + 1;
    }

    // if month has 30 days change next day and month to next month
    if (((currentMonth == 3) || (currentMonth == 5) || (currentMonth == 8) || (currentMonth == 10)) && (nextCurrentDay == 31))  // 30 days
    {
        nextCurrentDay = 1;
        nextCurrentMonth = nextCurrentMonth + 1;
    }

    // if leap year allow February to have 29 days
    if (((currentYear == 2024) || (currentYear == 2028) || (currentYear == 2032) || (currentYear == 2036)) && (currentMonth == 1) && (nextCurrentDay == 30))  // 29 days - leap years up to 2039
    {
        nextCurrentDay = 1;
        nextCurrentMonth = nextCurrentMonth + 1;
    }
    else if ((currentMonth == 1) && (nextCurrentDay == 29))  // go to next month if not a leap year
    {
        nextCurrentDay = 1;
        nextCurrentMonth = nextCurrentMonth + 1;
    }

    // if last day of year go to next year
    if (nextCurrentMonth == 12) {nextCurrentMonth = 0; nextCurrentYear = nextCurrentYear + 1;}  // end of year

    // get next midnight milliseconds based on above
    nextMidnightMillis = tmConvert_t(nextCurrentYear,nextCurrentMonth,nextCurrentDay,0,0,00);

    // std::cout << "-currentMillisTime  - " << currentMillisTime << std::endl;
    // std::cout << "-midnightMillis     - " << midnightMillis << std::endl;
    // std::cout << "-nextMidnightMillis - " << nextMidnightMillis << std::endl;
    // std::cout << "-currentDay           - " << currentDay << std::endl;
    // std::cout << "-currentMonth         - " << currentMonth << std::endl;
    // std::cout << "-currentYear          - " << currentYear << std::endl;
    // std::cout << "-nextCurrentDay       - " << nextCurrentDay << std::endl;
    // std::cout << "-nextCurrentMonth     - " << nextCurrentMonth << std::endl;
    // std::cout << "-nextCurrentYear      - " << nextCurrentYear << std::endl;
}

// long tz_offset(time_t when = LC_TIME)
// {
//     if (when == LC_TIME)
//         when = std::time(nullptr);
//     auto const tm = *std::localtime(&when);
//     std::ostringstream os;
//     os << std::put_time(&tm, "%z");
//     std::string s = os.str();
//     // s is in ISO 8601 format: "±HHMM"
//     int h = std::stoi(s.substr(0,3), nullptr, 10);
//     int m = std::stoi(s[0]+s.substr(3), nullptr, 10);    
//     UTCoffset = (h * 3600 + m * 60) * 1000;
//     connectMysql();
//     std::string query = "UPDATE dateTimeSync SET UTCoffset = '" + to_string(UTCoffset) + "' WHERE syncID = '0'";
//     mysql_query(conn, query.c_str());  // run query
//     mysql_disconnect();     
//     return true;
// }

void systemSettings::syncDateTimeError()
{
    // std::cout << "syncDateTimeError--------------------------------" << std::endl;
    MYSQL_RES *res;
    MYSQL_ROW row;

    long long offset = 0;
    // load current timezone and last time date/time was synced
    connectMysql();
    std::string query = "SELECT currentMillisLastSync FROM dateTimeSync WHERE syncID = '0'";  // construct query string
    mysql_query(conn, query.c_str());  // run query
    res = mysql_use_result(conn);
    while ((row = mysql_fetch_row(res)) != NULL)
    {
		currentMillisLastSync = std::stoll(row[0]);
    }
    mysql_disconnect(); 
    mysql_free_result(res);

    if (startupComplete == 0) {  // if no internet 
        // std::cout << "sync Start UP-----------------------" << std::endl;
        currentMillisLastSync = currentMillisLastSync;
    }

    // currentMillisLastSync = 1613128626985;

    currentMillisTracker = (currentMillisLastSync - currentMillisLastSync);  // update hardware current millis() counter based on how long since last sync
    currentMillisLastSync = (currentMillisLastSync + currentMillisTracker);  // updated unix current millis with

    // currentMillisTime = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();


    // std::cout << "currentMillisLastSync " << currentMillisLastSync << std::endl;
    // std::cout << "currentMillisTime       - " << currentMillisTime << std::endl;
    // std::cout << "currentMillisLastSync   - " << currentMillisLastSync << std::endl;

}

void systemSettings::syncDateTimeStartup()
{
    // std::cout << "syncDateTimeStartup-----------------------" << std::endl;
    if (system("ping -c1 -s1 www.google.com")) {
        // std::cout << "There is no internet connection  \n";
        // syncDateTimeError();
        MYSQL_RES *res;
        MYSQL_ROW row;

        long long offset = 0;
        // load current timezone and last time date/time was synced
        connectMysql();
        std::string query = "SELECT currentMillisLastSync FROM dateTimeSync WHERE syncID = '0'";  // construct query string
        mysql_query(conn, query.c_str());  // run query
        res = mysql_use_result(conn);
        while ((row = mysql_fetch_row(res)) != NULL)
        {
            currentMillisLastSync = std::stoll(row[0]);
        }
        mysql_disconnect(); 
        mysql_free_result(res);

        currentMillisOffset = currentMillisTracker;
        // currentMillisLastSync = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();
        // currentMillisTracker = millis();
        // currentMillisLastSync = currentMillisLastSync + (currentMillisTracker - currentMillisOffset);
        currentMillisTracker = millis();  // get current millis since program started
        currentMillisTime = currentMillisLastSync + (currentMillisTracker - currentMillisOffset);  // 
    }
    else {
        // syncDateTimeError();
        currentMillisTracker = millis();  // get current millis since program started
        currentMillisOffset = currentMillisTracker;
        currentMillisLastSync = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();
        currentMillisTracker = millis();  // get current millis since program started
        currentMillisTime = currentMillisLastSync + (currentMillisTracker - currentMillisOffset);  // 
        
        connectMysql();
        std::string query = "UPDATE dateTimeSync SET currentMillisLastSync = '" + to_string(currentMillisLastSync) + "'  WHERE syncID = '0'";
        mysql_query(conn, query.c_str());  // run query
        mysql_disconnect(); 

        // std::cout << "---------------- currentMillisOffset  - " << currentMillisOffset << std::endl;
        // std::cout << "---------------- currentMillisLastSync    - " << currentMillisLastSync << std::endl;
         
    }
    // prevTimeSyncMillis = currentMillisTime;
}

void systemSettings::syncDateTime()
{
    currentMillisTracker = millis();  // get current millis since program started
    currentMillisTime = currentMillisLastSync + (currentMillisTracker - currentMillisOffset);  // 

    // if (currentMillisTime >= (prevTimeSyncMillis + 1800000)) 
    if (currentMillisTime >= (prevTimeSyncMillis + 10000)) 
    {
        prevTimeSyncMillis = currentMillisTime;
        syncDateTimeStartup();
    }
}

std::string systemSettings::format_number(int num, int l) {  // used to add zeros in front of number
    ostringstream out;
    out << std::internal << std::setfill('0') << std::setw(l) << num;
    return out.str();
}

void systemSettings::restart()  // restart controller
{
    std::string comm = "sudo reboot";
    system(comm.c_str());
}

systemSettings systemSettings;