#!/bin/bash
# sudo apt-get install htop  // used to check CPU
# sudo bash /boot/install_robotank.sh


# check if the reboot flag file exists. 
# if [ ! -f /home/pi/resume-after-reboot ]; then
    echo "********************************************************************************"
    echo "********************************************************************************"
    echo "********************************************************************************"
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""
    echo "This script will update the Pi operating system and"
    echo "install Robo-Tank with the required packages to run it."
    echo "This will take 10 - 30 minutes to complete."
    # echo "During the installation the Raspberry Pi will reboot, following this restart the SSH terminal again so the installation process can be completed."

    echo ""
    echo "Double click the link below to open a list of time zones."
    echo ""
    echo "Visit https://en.wikipedia.org/wiki/List_of_tz_database_time_zones"
    echo ""
    echo "Find your location in the 2nd column 'TZ database name', "
    echo "type it in exactly with proper casing or you will receive an error."
    echo ""
    echo ""
    # echo "Press enter to continue or press Ctrl + Z to abort"
    # read input
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""

    # check if sudo is used
    if [ "$(id -u)" != 0 ]; then
        echo 'You need to run this script with sudo'
        exit 1
    fi
    
    # update timezone
    echo "Enter your Timezone and Press Enter"
    read timez
    echo ""
    echo ""
    echo "You set your timezone to " $timez
    sudo timedatectl set-timezone $timez
    # sudo timedatectl set-timezone America/Edmonton

    # # sync time
    # echo "Syncing time"
    sudo systemctl start systemd-timesyncd
    sudo systemctl enable systemd-timesyncd


    echo "Updating config.txt File"
    ################################## Edit config.txt file ##############################################
    # enable I2C
    if grep -q 'i2c-bcm2708' /etc/modules; then
        echo ""
    else
        echo 'i2c-bcm2708' >> /etc/modules
    fi
    if grep -q 'i2c-dev' /etc/modules; then
        echo ""
    else
        echo 'i2c-dev' >> /etc/modules
    fi
    if grep -q 'dtparam=i2c_arm=on' /boot/config.txt; then
        echo ""
    else
        echo 'dtparam=i2c_arm=on' >> /boot/config.txt
    fi
    sudo sed -i 's/#dtparam=i2c_arm=on/dtparam=i2c_arm=on/g' /boot/config.txt
    if [ -f /etc/modprobe.d/raspi-blacklist.conf ]; then
        sudo sed -i 's/^blacklist spi-bcm2708/#blacklist spi-bcm2708/' /etc/modprobe.d/raspi-blacklist.conf
        sudo sed -i 's/^blacklist i2c-bcm2708/#blacklist i2c-bcm2708/' /etc/modprobe.d/raspi-blacklist.conf
    else
        echo ""
    fi
    # echo "Disabling SPI"
    # # replace word in file
    sudo sed -i 's/dtparam=spi=on/dtparam=spi=off/g' /boot/config.txt

    # echo "Changing I2C speed to 10 kHz"
    # # # add line after search
    sudo sed -i '/dtparam=spi=off/a dtparam=i2c_arm_baudrate=10000' /boot/config.txt
    sudo sed -i '/dtparam=i2c_arm_baudrate=10000/a #dtoverlay=w1-gpio' /boot/config.txt

    # echo "Enabling 1-wire"
    # # # replace word in file
    sudo sed -i 's/#dtoverlay=w1-gpio/dtoverlay=w1-gpio/g' /boot/config.txt

    # echo "Enabling PWM"
    # # # add line after search
    sudo sed -i '/dtoverlay=w1-gpio/a dtoverlay=pwm-2chan' /boot/config.txt
 


    echo "Updating Raspberry Pi Operating System    ...takes 1-20 minutes"
    sudo apt update &> /dev/null
    sudo apt full-upgrade -y &> /dev/null
    sudo apt autoremove -y &> /dev/null


    echo 'Installing i2c-tools                      ...takes less than 1 minute'
    sudo apt-get install -y i2c-tools &> /dev/null


    echo "Installing Apache2                        ...takes about 1 minute" 
    sudo apt install apache2 -y &> /dev/null
    sudo a2enmod rewrite &> /dev/null
    sudo systemctl restart apache2 &> /dev/null
    sudo chown -R pi:www-data /var/www/html/
    sudo chmod -R 770 /var/www/html/
    sudo systemctl restart apache2 &> /dev/null

    #copy updated apache2.cong file
    sudo cp /boot/robotank/install/apache2.conf /etc/apache2/apache2.conf

    # sudo systemctl restart apache2 &> /dev/null
    
    #copy updated 99-com.rules file
    sudo cp /boot/robotank/install/99-com.rules /etc/udev/rules.d/99-com.rules 


    echo "Installing PHP                            ...takes about 1 minute"
    sudo apt install php libapache2-mod-php -y &> /dev/null
    sudo rm /var/www/html/index.html &> /dev/null
    
    # create test PHP file
    # echo "<?php phpinfo ();?>" > /var/www/html/index.php


    echo "Installing MySQL                          ...takes about 2 minute"
    sudo apt install mariadb-server php-mysql -y &> /dev/null
    
    sudo /usr/share/debconf/fix_db.pl  # run database fix as recommended
    sudo mysql --user=root < /boot/robotank/install/create_database.sql &> /dev/null  # create database user


    echo "Installing PHPMyAdmin                     ...takes about 2 minute"

    echo "mysql-server mysql-server/root_password password robotank" | sudo debconf-set-selections
    echo "mysql-server mysql-server/root_password_again password robotank" | sudo debconf-set-selections
    echo 'phpmyadmin phpmyadmin/dbconfig-install boolean false' | sudo debconf-set-selections
    echo 'phpmyadmin phpmyadmin/app-password-confirm password robotank' | sudo debconf-set-selections
    echo 'phpmyadmin phpmyadmin/mysql/admin-pass password robotank' | sudo debconf-set-selections
    echo 'phpmyadmin phpmyadmin/mysql/app-pass password robotank' | sudo debconf-set-selections
    echo 'phpmyadmin phpmyadmin/reconfigure-webserver multiselect apache2' | sudo debconf-set-selections


    sudo apt -y install phpmyadmin php-mbstring php-gettext &> /dev/null
    sudo phpenmod mysqli &> /dev/null
    sudo /etc/init.d/apache2 restart &> /dev/null
    # sudo rm /var/www/html/index.php /dev/null
    sudo mysql --user=root < /boot/robotank/install/create_database.sql &> /dev/null  # create database user
    sudo /etc/init.d/apache2 restart &> /dev/null

    # update phpmyadmin, fixes error on import and export tabs
    sudo sed -i '/if ($options != null && count($options) > 0)/c\if (! is_null($options) && count((array)$options) > 0) {' /usr/share/phpmyadmin/libraries/plugin_interface.lib.php

    
    echo "Installing Robo-Tank                      ...takes less than 1 minute"
    sudo cp -r /boot/robotank/* /var/www/html &> /dev/null
    sudo chown -R pi:www-data /var/www/html/ &> /dev/null
    
    
    echo "Installing Robo-Tank Database             ...takes less than 1 minute"
    sudo mysql --user=root --password=robotank < /boot/robotank/install/robotank_database.sql &> /dev/null

    # fix bug in phpmyadmin
    sudo cp /boot/robotank/install/sql.lib.php /usr/share/phpmyadmin/libraries/sql.lib.php &> /dev/null


    echo "Installing JSON Library                   ...takes less than 1 minute"
    sudo apt-get install jq -y &> /dev/null


    # copy updated one wire configuration file so ds18b20 is found quicker
    sudo cp /boot/robotank/install/w1.conf /etc/modprobe.d/w1.conf
    sudo cp /boot/robotank/install/modules.conf /etc/modules.conf


    # copy updated php.ini file, set session to 86400 seconds and cache to 60 seconds.
    sudo cp /boot/robotank/install/php.ini /etc/php/7.3/apache2/php.ini


    echo "Installing MySQL Library                  ...takes less than 1 minute"
    sudo apt-get install libmariadb-dev-compat libmariadb-dev -y &> /dev/null


    # echo "Enabling CGI processing in apache"
    sudo a2enmod cgi &> /dev/null


    # echo "Enabling .htaccess headers"
    sudo a2enmod headers &> /dev/null


    # # enable .htaccess ExpiresActive
    sudo a2enmod expires &> /dev/null
    sudo systemctl restart apache2 &> /dev/null


    echo "Installing wiringPi Library               ...takes less than 1 minute"
    wget -q https://project-downloads.drogon.net/wiringpi-latest.deb &> /dev/null
    sudo dpkg -i wiringpi-latest.deb &> /dev/null
    sudo rm wiringpi-latest.deb &> /dev/null


    echo "Installing libwebsockets Library          ...takes less than 1 minute"
    sudo apt-get install libwebsockets-dev -y &> /dev/null


    echo "Installing JsonCpp Library                ...takes less than 1 minute"
    sudo apt-get install libjsoncpp-dev -y &> /dev/null


    echo "Installing libcurl Library                ...takes less than 1 minute"
    sudo apt-get install libcurl4-openssl-dev -y &> /dev/null


    echo "Installing libquickmail Library           ...takes about 1 minute"
    cd /boot/robotank/install/libquickmail/  # change directory
    sudo ./configure &> /dev/null  # configure libquickmail
    sudo make &> /dev/null  # compile libquickmail
    sudo make install &> /dev/null  # install libquickmail
    sudo sh -c "echo '/usr/local/lib' >> /etc/ld.so.conf"  # add path to installed 3rd party libraries
    sudo ldconfig &> /dev/null  # refresh cache
    # sudo cp /boot/robotank/install/libquickmail/quickmail.h /usr/local/include/  # MAYBE
    
    
    echo "Updating Raspberry Pi Operating System    ...takes 1-15 minutes"
    sudo apt update &> /dev/null
    sudo apt full-upgrade -y &> /dev/null
    sudo apt autoremove -y &> /dev/null


    echo "Installing Robo-Tank Service on Pi        ...takes less than 1 minute"
    sudo cp /boot/robotank/install/robotank.service /etc/systemd/system/robotank.service &> /dev/null

    # give Robo-Tank service file permission
    sudo chmod -R 770 /etc/systemd/system/robotank.service &> /dev/null

    # enable Robo-Tank service to run at startup
    sudo systemctl enable robotank.service &> /dev/null
    sudo systemctl start robotank.service &> /dev/null
    # sudo systemctl stop robotank.service
    # sudo systemctl disable robotank.service
    
    echo "Clean up Robo-Tank Installation Files     ...takes less than 1 minute"
    sudo rm -r /boot/robotank/ &> /dev/null

    echo ""
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""
    echo "-------------------------------------------------------------------------------------------------------------------------"
    echo ""
    echo "Installation Complete"
    echo ""
    echo "-------------------------------------------------------------------------------------------------------------------------"
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""

    # echo "DROP USER 'root'@'localhost';"
    # echo "CREATE USER 'root'@'localhost' IDENTIFIED BY 'robotank';"
    # echo "GRANT ALL PRIVILEGES ON *.* TO 'root'@'localhost' WITH GRANT OPTION;"
 
    # sudo mysql --user=root
    echo "The Raspberry Pi needs to be restarted."
    echo ""
    echo ""
    echo "After the Pi restarts open a web browser on a device connected to your network"
    echo "and type in the following address to open the Robo-Tank App"
    echo ""
    ip= hostname -I | awk '{ print "http://" $1 }'
    echo ""
    echo ""
    echo ""
    echo ""
    echo "The default login for Robo-Tank is as follows."
    echo ""
    echo "Username = Robo-Tank"
    echo "Password = Robo-Tank"
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""
    echo "Press enter to restart the Raspberry Pi..."
    echo ""
    echo ""
    read input
    

    echo 'Expanding file system'
    # # # expand file system
    sudo raspi-config --expand-rootfs &> /dev/null

    sudo reboot
# fi
